#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé le Sun May  8 16:45:40 2022

@auteur: david ALBERTO
(www.astrolabe-science.fr)
"""
import matplotlib.pyplot as plt # graphique
import numpy as np # gestion de listes de données
import pandas as pd # gestion et import de tableaux de données

# Paramètres utilisateur :

taille_pt = 6
graduations = True
couleur_stables = 'indigo'
couleur_instables = 'forestgreen'
couleur_ax2 = 'saddlebrown'
c_fond = 'wheat' # couleur arrière-plan

# mise en forme graphique :
prop_stables = dict(color = couleur_stables, ec=None, alpha=1, lw=0, s=taille_pt)
prop_instables = dict(color = couleur_instables, ec=None, alpha=0.6, lw=0,
                      s=taille_pt)

plt.rcParams["font.family"] = "Roboto Condensed" # ou autre police installée

# --------------------------------------
# Lecture du fichier des énergies de liaison par nucléon:
donnees = pd.read_csv("data_binding_energy_per_nucleon.csv",
                    skiprows=26,
                    sep="\t",
                    usecols=[1,2,4],
                    names=['A','Z', 'E_liaison']
                    )
donnees = donnees.loc[donnees['E_liaison'] >= 0] #élimine les valeurs négatives
effectif = len(donnees)

# import des données d'identification des noyaux stables :
data_stables = pd.read_csv('noyaux_stables.csv', sep=';')

def est_stable(Z,A):
    """
    Parameters
    ----------
    Z : TYPE : (entier) : numéro atomique
    A : TYPE : (entier) : nombre de masse
    Returns
    -------
    renvoie True si le nucléide est stable, False si radioactif.
    """
    N = A - Z
    stable = False
    for i in range(len(data_stables)):
        if data_stables.Z[i] == Z and data_stables.N[i] == N:
            stable = True
    return stable

def colonne_stabilite(df):
    """
    df : (DataFrame) : tableau de données isotopiques
    Returns
    -------
    ajoute une colonne dans le tableau df, indiquant "True" si le noyau
    est stable, "False" sinon.
    """
    stable = []
    for i in range(len(df)):
        Z = df.iloc[i].Z
        A = df.iloc[i].A
        if est_stable(Z, A):
            stable.append(True)
        else:
            stable.append(False)
    df2 = df
    df2['stable'] = stable
    return df2

tableau = colonne_stabilite(donnees)

A = tableau['A']
E_liaison = tableau['E_liaison']

#Récupération du nombre de masse ayant l'énergie maximale :
Emax = max(E_liaison)
Amax = tableau.loc[tableau['E_liaison'] == Emax, 'A'].values[0]

# -----------------------------------------
# PARAMETRES DU GRAPHIQUE
fig = plt.figure(
    figsize=(29.7/2.54,21/2.54), # format A4
    # figsize=(21/2.54,29.7/2/2.54), # format A5
    tight_layout=True,
               facecolor=c_fond)

#---------------------------------------------
#  GRAPHE PRINCIPAL
ax = plt.subplot(111)

ax.set_xlabel("nombre de masse A")
ax.set_ylabel("Énergie de liaison par nucléon (MeV/nucléon)")
ax.set_title("Courbe d'Aston", fontsize=16, y=1.01)
ax.set_xlim(0,300)
ax.set_xticks(np.arange(0,300+20,20))
ax.set_xticks(np.arange(0,300+5,5), minor=True)
ax.set_ylim(0,9)
ax.set_yticks(np.arange(0,9+1,1))
ax.set_yticks(np.arange(0,9+0.5,0.5), minor=True)


tableau = tableau.sort_values(by='A')# tri selon A croissant

# tracé des noyaux stables :
A_stables = tableau.loc[tableau.stable == True, 'A']
E_liaison_stables = tableau.loc[tableau.stable == True, 'E_liaison']
ax.scatter(A_stables,E_liaison_stables, **prop_stables)
ax.plot(A_stables,E_liaison_stables, c=couleur_stables,alpha=0.8, label='stables')

# tracé des noyaux instables :
A_instables = tableau.loc[tableau.stable == False, 'A']
E_liaison_instables = tableau.loc[tableau.stable == False, 'E_liaison']
ax.scatter(A_instables,E_liaison_instables, **prop_instables,
           label='radioactifs')

ax.legend()

if graduations==True:
    ax.grid(which='minor',
            alpha=0.5
            )
    ax.grid(which='major',
            lw=1, zorder=0
            )
    ax.set_axisbelow(True)

# affichage du noyau le plus stable :
ax.scatter(Amax,Emax,s=taille_pt, c='mediumblue', zorder=2)
ax.text(Amax,Emax+0.05,f" A={Amax}", fontsize=8, c='mediumblue',
        ha='center')

effectif_stables = len(A_stables)

ax.text(0.01,0.01,f'{effectif} noyaux', ha='left', transform=ax.transAxes)
ax.text(0.01,0.04,f'{effectif_stables} noyaux stables' , ha='left',
        c=couleur_stables, transform=ax.transAxes)

ax.set_axisbelow(True) # grille sous les tracés
ax.grid(which='minor',
        alpha=0.5
        )
ax.grid(which='major',
        lw=1)

texte1='D. Alberto (www.astrolabe-science.fr)'
texte2='données : Atomic Mass Evaluation 2016 Chinese Physics C 41, 030002 (2016),\n NuBase 2016 Chinese Physics C 41, 030001 (2016)'
ax.text(1.01,0.0,texte1+'     '+texte2,
        transform=ax.transAxes, rotation=90,
        fontsize=6.5)
#  ---------------------------------------------------------------------
#  2e graphique inséré dans le 1er :
ax2 = ax.inset_axes([0.5,0,0.5,0.5])# insérer un axe
ax2.set(
        # xticks=([]),
        # yticks=([]),
        xlim=(0,30),
        ylim=(0,9),
        xticks=np.arange(5,30,5),
        yticks=np.arange(1,9,1),
        )

#  cadre coloré :
ax2.spines['top'].set_color(couleur_ax2)
ax2.spines['left'].set_color(couleur_ax2)

ax2.tick_params(axis="x",direction="in", pad=-12, color=couleur_ax2,
                labelcolor = couleur_ax2)
ax2.tick_params(axis="y",direction="in", pad=-10, color=couleur_ax2,
                labelcolor = couleur_ax2)

# tracé des noyaux stables :
indexlim = 25 # nombre de lignes du tableau à conserver pour le 2e graphe
A_stables = A_stables[:indexlim]
E_liaison_stables = E_liaison_stables[:indexlim]
ax2.scatter(A_stables,E_liaison_stables, **prop_stables)
ax2.plot(A_stables,E_liaison_stables, c=couleur_stables,alpha=0.8, label='stables')

#  étiquetage des noyaux remarquables :
noyaux = [('He', 4),('C', 12), ('O', 16),('Ne',20)]
for i, (symbole, A) in enumerate(noyaux):
    x = noyaux[i][1]
    index = list(A_stables).index(x)
    y = E_liaison_stables.iloc[index]
    ax2.text(x,y+0.1, f"{noyaux[i][0]}-{A}", ha='center',
             c=couleur_stables)



fig.savefig("diagramme_Aston.pdf")
fig.savefig("diagramme_Aston.png", dpi=200)
